import validator from "validator";
import {omit, isEqual, isEmpty, isBoolean, split, trim, isNumber, includes} from 'lodash'
import "whatwg-fetch";
import fetch from 'isomorphic-fetch'
import * as numeral from 'numeral';

import t from './language';

export function arrowUpDown(key, dataIndex, data) {
    let x = null;

    if (key === 'ArrowUp') {
        if (dataIndex === null) {
            x = null
        } else if (dataIndex === 0) {
            x = 0
        } else {
            x = dataIndex - 1
        }
    } else if (key === 'ArrowDown') {
        if (dataIndex === null) {
            x = 0
        } else if (dataIndex !== data.length - 1 || dataIndex === 0) {
            x = dataIndex + 1
        } else if (dataIndex === data.length - 1) {
            x = data.length - 1
        }
    }

    return x
}

export function changeProps(nextProps, propsArr, state, callback) {
    propsArr.forEach((item) => {
        if (!isEqual(nextProps[item], state[item])) {
            callback({
                [item]: nextProps[item]
            }, item)
        }
    });
}

export function searchDataFetch(url, val, callback) {

    callback({
        data: [],
        count: 0,
        loading: true
    });

    fetch(url, {
        method: 'GET',
        headers: {"Accept": "application/json", "Content-Type": "application/json"}
    })
        .then(response => response.json())
        .then(response => {
            if (response.length) {
                callback({
                    data: val ? response : [],
                    count: response.length,
                    loading: false
                })
            } else {
                callback({
                    data: [],
                    count: 0,
                    loading: false
                })
            }
        })
        .catch(err => {
            console.log('Fetch Error', err)
        })
}


function abortRead() {
    reader.abort();
}

function errorHandler(evt) {
    switch (evt.target.error.code) {
        case evt.target.error.NOT_FOUND_ERR:
            alert('File Not Found!');
            break;
        case evt.target.error.NOT_READABLE_ERR:
            alert('File is not readable');
            break;
        case evt.target.error.ABORT_ERR:
            break; // noop
        default:
            alert('An error occurred reading this file.');
    }
}

export async function handleFileSelect(evt, error, cbStart, cbRead, cbDone, cbProgress) {

    const file = evt;

    let fileInfo = {
        name: file.name,
        type: file.type,
        id: file.lastModified,
        size: convertBytesToSize(file.size),
    };

    if (error) {

        cbStart({
            ...fileInfo,
            status:0,
            progress:0,
            error: error
        })

        cbDone({
            ...fileInfo,
            status: -1,
            read: false,
            output: null,
        })

        return
    }

    let progress = null;

    let reader = new FileReader();
    reader.onerror = errorHandler;
    reader.onabort = function(e) {
        console.log('Başarısız')
    };
    reader.onprogress = function(e) {
        if (e.lengthComputable) {

            let percentLoaded = Math.round((e.loaded / e.total) * 100);
                progress = percentLoaded < 100 ? percentLoaded : 100;

            cbProgress({
                id: file.lastModified,
                progress: progress,
            })
        }
    };
    reader.onloadstart = function(e) {
        cbStart({
            ...fileInfo,
            status:0,
        })
    };

    reader.onload = function(e) {

        cbRead({
            ...fileInfo,
            status: 1,
        })
    };

    reader.onloadend = function(e) {

        cbDone({
            ...fileInfo,
            status: 2,
            read: true,
            output: reader.result
        })
    };

    if (includes(file.type,'image')) {
        reader.readAsDataURL(file);
    } else {
        reader.readAsArrayBuffer(file);
    }

}

export function convertBytesToSize(bytes) {
    return {
        textValue: numeral(bytes).format('0.0 b'),
        value: bytes,
        size: numeral(bytes).format('0.0 b').split(' ')[0],
        unit: numeral(bytes).format('0.0 b').split(' ')[1],
    };
}

export function validateFunc(validateArr, val, msg, lang) {

    let errors = [];
    if (isBoolean(validateArr)) {
        validateArr = ['required']
    }

    if (!msg && validateArr && validateArr.length) {
        validateArr.forEach(item => {
            switch (item) {
                case 'required':
                    if (validator.isEmpty(val)) {
                        errors.push(t[lang].input.errRequired) // Bu alan gereklidir
                    }
                    break;
                case 'email':
                    if (!validator.isEmail(val)) {
                        errors.push(t[lang].input.errEmail)
                    }
                    break;
                case 'alpha':
                    if (!validator.isAlpha(val)) {
                        errors.push(t[lang].input.errAlpha)
                    }
                    break;
                case 'alphanumeric':
                    if (!validator.isAlphanumeric(val)) {
                        errors.push('Rakam ve harf içermelidir')
                    }
                    break;
                case 'numeric':
                    if (!validator.isNumeric(val)) {
                        errors.push('Sadece rakam girilebilir')
                    }
                    break;
                case 'number':
                    if (!validator.isNumeric(val)) {
                        errors.push('Sadece rakam girilebilir')
                    }
                    break;
                case 'creditcard':
                    if (!validator.isCreditCard(val)) {
                        errors.push('Bir kredi kartı numarası olmalıdır')
                    }
                    break;
                case 'url':
                    if (!validator.isURL(val, ['http', 'https', 'ftp'])) {
                        errors.push('Bir URL adres olmalıdır')
                    }
                    break;
                case 'time':

                    const pattern12 = new RegExp(/^(([0-1][0-1]):([0-5][0-9]))(:[0-5][0-9])?$/);
                    const res12 = pattern12.test(val);

                    if (!res12) {
                        errors.push('12\'lik bir saat formatı olmalıdır')
                    }
                    break;
                case 'time24':

                    const pattern24 = new RegExp(/^(([0-2][0-3]):([0-5][0-9]))(:[0-5][0-9])?$/);
                    const res24 = pattern24.test(val);

                    if (!res24) {
                        errors.push('24\'lük saat formatı olmalıdır')
                    }
                    break;
                default:
                    errors = []
            }
        });
    } else {
        errors.push(msg)
    }

    return errors
}

export function getId(id, uuid) {
    if (!id) id = uuid;
    return id;
}

export function nullToString(value) {
    let newValue = value;

    if (value === null || typeof value === "undefined" || typeof value === 'object' || typeof value === 'boolean') {
        newValue = '';
    } else {
        newValue = String(newValue)
    }

    return newValue
}

export function keyPressStringSTop(e) {
    const keycode = e.keyCode;
    const ctrlKey = e.ctrlKey;

    if ((keycode === 65 && ctrlKey) || (keycode === 67 && ctrlKey) || (keycode === 86 && ctrlKey) || (keycode === 88 && ctrlKey)) {
        return;
    } else if ((keycode > 64 && keycode < 91) || (keycode > 185 && keycode <= 188) || (keycode === 189 && keycode === 191 && keycode === 192) || (keycode > 218 && keycode < 223)) {
        e.preventDefault();
    }

    return e.key;
}

export function inputSetString(state, val) {
    let {dataIndex, selectData, dataShowFields, dataInputSetFields} = state,
        showStr = '',
        selectStr,
        setStr;

    if (val && !isEmpty(val)) {
        split(dataShowFields, ',').forEach(obj => {
            selectStr = val ? val[trim(obj)] : selectData[dataIndex][trim(obj)];
            showStr = showStr + ' ' + selectStr
        });

        if (dataInputSetFields && !isEmpty(dataInputSetFields)) {
            showStr = '';
            split(dataInputSetFields, ',').forEach(k => {
                setStr = val ? val[trim(k)] : selectData[dataIndex][trim(k)];
                showStr = showStr + ' ' + setStr
            });
        }
    }

    return trim(showStr)
}

